@@ -96,6 +96,30 @@ class TimeStamp:
          (self.hours, self.minutes, self.seconds, self.millisecods)
 
 
+class SectionMarker:
+    def __init__(self, arg):
+        try:
+            self.marker = TimeStamp(arg)
+        except Exception:
+            try:
+                self.marker = int(arg)
+            except Exception:
+                panic('Invalid section marker argument', 1)
+
+    def include_after(self, other):
+        if type(self.marker) is TimeStamp:
+            return other.time_start >= self.marker
+        return other.number >= self.marker
+
+    def include_before(self, other):
+        if type(self.marker) is TimeStamp:
+            return other.time_end <= self.marker
+        return other.number <= self.marker
+
+    def __le__(self, other):
+        return int(self) <= int(other)
+
+
 class Subtitle:
     # Parse a single subtitle
     def __init__(self, lines, file_name, line_number):
@@ -160,12 +184,12 @@ class ConfigFile:
     def __init__(self, args):
         # No reason to continue
         if not args.clean:
-            if args.config_file:
-                args.config_file.close()
+            if args.config:
+                args.config.close()
             self.expressions = []
             return
 
-        file = args.config_file
+        file = args.config
         # Set default config file if not specified
         if not file:
             home = Path.home()
@@ -235,20 +259,10 @@ class SubripFile:
         self.subs += other.subs
         return self
 
-    def clean(self, expressions):
-        if len(expressions) == 0:
-            return
-
-        # Remove lines matching any expression
-        for regexp in expressions:
-            subs = filter(lambda sub: not sub.matches(regexp), self.subs)
-
-        self.subs = list(subs)
-
     def shift(self, ms):
         for sub in self.subs:
             sub.shift(ms)
-        self.subs = list(filter(lambda sub: sub.time_start >= 0, self.subs))
+        self.subs = [sub for sub in self.subs if sub.time_start >= 0]
 
     def strip_html(self):
         p = re.compile('<.+?>')
@@ -262,25 +276,71 @@ class SubripFile:
             i += 1
 
     def process(self, args, config):
-        if args.clean:
-            self.clean(config.expressions)
+        html_regex = re.compile('<.+?>')
+        new_subs = []
+        for sub in self.subs:
+            if args.begin and not args.begin.include_after(sub):
+                new_subs.append(sub)
+                continue
 
-        if args.shift:
-            self.shift(args.shift)
+            if args.end and not args.end.include_before(sub):
+                new_subs.append(sub)
+                continue
 
-        if args.no_html:
-            self.strip_html()
+            if args.clean and len(config.expressions) > 0:
+                if any(sub.matches(regex) for regex in config.expressions):
+                    continue
 
+            if args.shift:
+                sub.shift(args.shift)
+                if sub.time_start < 0:
+                    continue
+
+            if args.no_html:
+                sub.replace(html_regex, '')
+
+            new_subs.append(sub)
+
+        self.subs = new_subs
+
+        self.write_file(args.replace)
+
+    def write_file(self, in_place=False, stdout=False):
         self.renumber()
-        self.write_file()
 
-    def write_file(self):
-        output = open(self.file_name, 'w', encoding='utf-8')
+        if stdout:
+            print(self)
+            return
+
+        try:
+            if in_place:
+                path = self.file_name
+                output = open(path, 'w', encoding='utf-8')
+            else:
+                path = Path(self.file_name)
+                path = path.with_name('out-' + path.name)
+                output = path.open(mode='w', encoding='utf-8')
+        except PermissionError:
+            panic('Can\'t access file {}: Permission denied'
+                  .format(path), 1)
+
         output.write(repr(self))
+
         if len(self.subs) > 0:
             output.write('\n')
+
         output.close()
 
+    def delete(self):
+        os.remove(self.file_name)
+        del self
+
+    def trunc_before(self, marker):
+        self.subs = [sub for sub in self.subs if marker.include_after(sub)]
+
+    def trunc_after(self, marker):
+        self.subs = [sub for sub in self.subs if marker.include_before(sub)]
+
     def __repr__(self):
         return '\n\n'.join(map(repr, self.subs))
 
@@ -292,7 +352,23 @@ def parse_args(args):
         add_help=True
     )
 
+    # Requires --clean
     parser.add_argument(
+        '-f', '--config',
+        help='use F as the config file (by default, on Unix it is: "$HOME/' +
+             r'.config/fsubrc"; on Windows it is: "%%APPDATA%%\fsubrc")',
+        metavar='F',
+        action='store',
+        type=argparse.FileType('r')
+    )
+
+    processing = parser.add_argument_group(
+        'processing',
+        'Flags that specify an action to be taken. Many may ' +
+        'be specified.'
+    )
+
+    processing.add_argument(
         '-c', '--clean',
         help='remove subtitles matching regular expressions ' +
              'listed in the config file (this is the default ' +
@@ -300,7 +376,7 @@ def parse_args(args):
         action='store_true'
     )
 
-    parser.add_argument(
+    processing.add_argument(
         '-s', '--shift',
         help='shift all subtitles by MS milliseconds, which ' +
              'may be positive or negative',
@@ -309,25 +385,37 @@ def parse_args(args):
         type=int
     )
 
-    parser.add_argument(
+    processing.add_argument(
         '-n', '--no-html',
         help='strip HTML tags from subtitles content',
         action='store_true'
     )
 
-    parser.add_argument(
-        '-f', '--config-file',
-        help='overwrite the default config file (Unix: $HOME/.config/fsubrc,' +
-             r' Windows: %%APPDATA%%\fsubrc)',
-        metavar='FILE',
-        action='store',
-        type=argparse.FileType('r')
+    processing.add_argument(
+        '-j', '--join',
+        help='join all files into the first, shifting their time accordingly',
+        action='store_true'
     )
 
-    parser.add_argument(
-        '-j', '--join',
-        help='join all files into the first, shifting their time ' +
-             'accordingly (this will delete files)',
+    # Requires --begin or --end, may have both
+    processing.add_argument(
+        '-u', '--cut-out',
+        help='cut the specified section from the file(s) into new files',
+        action='store_true'
+    )
+
+    redirection = parser.add_mutually_exclusive_group()
+    redirection.add_argument(
+        '-r', '--replace',
+        help='edit files in-place (--join will delete joined files too), ' +
+             'instead of the default behavior of outputing results into ' +
+             'files prefixed with "out-"',
+        action='store_true'
+    )
+
+    redirection.add_argument(
+        '-p', '--stdout',
+        help='dump results to stdout, and do not edit nor write any file',
         action='store_true'
     )
 
@@ -339,16 +427,51 @@ def parse_args(args):
         nargs='+'
     )
 
+    section = parser.add_argument_group(
+        'sectioning',
+        'Flags that specify a section to work in. They accept either ' +
+        'a subtitle number or a time stamp in the SubRip format ' +
+        '("<hours>:<minutes>:<seconds>,<milliseconds>", where hours, minutes' +
+        ', seconds are 2-zero padded while milliseconds is 3-zero padded).' +
+        ' fsub will not modify subtitles outside this range, except while ' +
+        'joining the files.'
+    )
+
+    section.add_argument(
+        '-b', '--begin',
+        help='specify section beginning (inclusive)',
+        metavar='B',
+        action='store'
+    )
+
+    section.add_argument(
+        '-e', '--end',
+        help='specify section end (inclusive)',
+        metavar='E',
+        action='store'
+    )
+
     args = parser.parse_args(args)
 
+    # Flags that require section
+    if args.cut_out:
+        if not args.begin and not args.end:
+            panic('You must specify a section to work with', 1)
+
     # Make sure --clean is the default
-    if not args.shift and not args.no_html:
+    if not any((args.shift, args.no_html, args.join, args.cut_out)):
         args.clean = True
 
     # Validate options
-    if not args.clean and args.config_file:
+    if not args.clean and args.config:
         panic('-f requires -c', 1)
 
+    if args.begin:
+        args.begin = SectionMarker(args.begin)
+
+    if args.end:
+        args.end = SectionMarker(args.end)
+
     return args
 
 
@@ -360,11 +483,22 @@ def run(args):
     for file in args.files:
         parsed_files.append(SubripFile(file))
 
+    if args.cut_out:
+        if args.begin:
+            for file in parsed_files:
+                file.trunc_before(args.begin)
+
+        if args.end:
+            for file in parsed_files:
+                file.trunc_after(args.end)
+
     if args.join:
         first = parsed_files.pop(0)
         while True:
             try:
-                first += parsed_files.pop(0)
+                file = parsed_files.pop(0)
+                first += file
+                file.delete()
             except IndexError:
                 break
         parsed_files.append(first)
